<?php

/**
 * @file
 * This file contains all Drupal hooks for theming content.
 */

/**
 * Implements hook_preprocess_HOOK().
 *
 * Pre-rendre the table of available Chado connections.
 *
 * @ingroup tripal_mc
 */
function tripal_mc_preprocess_tripal_mc_admin(&$variables) {
  // Set the breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('Home'), '<front>');
  $breadcrumb[] = l(t('Administration'), 'admin');
  $breadcrumb[] = l(t('Tripal'), 'admin/tripal');
  $breadcrumb[] = l(t('Extensions'), 'admin/tripal/extension');
  $breadcrumb[] = l(t('Multi-Chado'), 'admin/tripal/extension/tripal_mc');
  drupal_set_breadcrumb($breadcrumb);

  $variables['connection_table'] = entity_ui_controller(TRIPAL_MC_CONNECTION_ENTITY_TYPE)->overviewTable();
  $variables['pager'] = theme('pager');
}

/**
 * Implements hook_preprocess_HOOK().
 *
 * Pre-rendre the table of available Chado connections.
 *
 * @ingroup tripal_mc
 */
function tripal_mc_preprocess_tripal_mc_schema_admin(&$variables) {
  // Set the breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('Home'), '<front>');
  $breadcrumb[] = l(t('Administration'), 'admin');
  $breadcrumb[] = l(t('Tripal'), 'admin/tripal');
  $breadcrumb[] = l(t('Extensions'), 'admin/tripal/extension');
  $breadcrumb[] = l(t('Multi-Chado'), 'admin/tripal/extension/tripal_mc');
  drupal_set_breadcrumb($breadcrumb);

  $schema_table = array();

  $schemata = tripal_mc_get_available_chado_schemata();
  $rows = array();
  if (!empty($schemata)) {
    foreach ($schemata as $schema_name) {
      $sql_query = "
        SELECT
          SUM(
            pg_total_relation_size(
              quote_ident(schemaname)
              || '.'
              || quote_ident(tablename)
            )
          )::BIGINT AS \"size\"
        FROM pg_tables
        WHERE schemaname = :schema;
      ";
      $result = db_query($sql_query, array(':schema' => $schema_name));
      $size = $result->fetchAssoc()['size'];
      if ($size > 1048576) {
        $size = round($size / 1048576.) . t('MB');
      }
      else {
        $size = $size . t('B');
      }
      if ('chado' == $schema_name) {
        $rows[] = array(
          $schema_name,
          $size,
          t('Default Chado schema'),
        );
      }
      else {
        $rows[] = array(
          $schema_name,
          $size,
          l(
            t('Delete'),
            'admin/tripal/extension/tripal_mc/schema/'
            . $schema_name
            . '/delete'
          ),
        );
      }
    }

    // Sort results and let 'chado' be first.
    usort($rows, function ($a, $b) {
      if (('chado' == $a[0]) && ('chado' == $b[0])) {
        return 0;
      }
      else if ('chado' == $a[0]) {
        return -1;
      }
      else if ('chado' == $b[0]) {
        return 1;
      }
      else {
        return strcmp($a[0], $b[0]);
      }
    });

    $schema_table = array(
      'header' => array(
        t('Schema name'),
        t('Schema size'),
        t('Action'),
      ),
      'rows' => $rows,
      'attributes' => array(),
      'caption' => t('Available Chado schemata'),
      'colgroups' => array(),
      'sticky' => TRUE,
      'empty' => t('No Chado schema found!'),
    );
      
  }

  $variables['schema_table'] = $schema_table;
}
